@@ -20,8 +20,8 @@ from accerciser.i18n import _, N_
 import pango
 from gtk import keysyms, gdk
 
-GLADE_FILE = os.path.join(os.path.dirname(__file__), 
-                          'event_monitor.glade')
+UI_FILE = os.path.join(os.path.dirname(__file__), 
+                       'event_monitor.ui')
 
 class EventMonitor(ViewportPlugin):
   '''
@@ -31,8 +31,8 @@ class EventMonitor(ViewportPlugin):
   Either source_app and source_acc for selected applications and accessibles 
   respectively. Or everything.
   @type source_filter: string
-  @ivar main_xml: The main event monitor glade file.
-  @type main_xml: gtk.glade.XML
+  @ivar main_xml: The main event monitor gtkbuilder file.
+  @type main_xml: gtk.GtkBuilder
   @ivar monitor_toggle: Toggle button for turining monitoring on and off.
   @type monitor_toggle: gtk.ToggleButton
   @ivar listen_list: List of at-spi events the monitor is currently listening
@@ -68,18 +68,21 @@ class EventMonitor(ViewportPlugin):
                             self._onClearlog,
                             keysyms.t, gdk.MOD1_MASK | gdk.CONTROL_MASK)]
     self.source_filter = None
-    self.main_xml = gtk.glade.XML(GLADE_FILE, 'monitor_vpaned')
-    vpaned = self.main_xml.get_widget('monitor_vpaned')
+    self.main_xml = gtk.Builder()
+    self.main_xml.add_from_file(UI_FILE)
+    vpaned = self.main_xml.get_object('monitor_vpaned')
     self.plugin_area.add(vpaned)
-    self._initTreeView()
+    self.events_model = self.main_xml.get_object('events_treestore')
     self._popEventsModel()
     self._initTextView()
 
-    self.monitor_toggle = self.main_xml.get_widget('monitor_toggle')
+    self.monitor_toggle = self.main_xml.get_object('monitor_toggle')
 
     self.listen_list = []
 
-    self.main_xml.signal_autoconnect(self)
+    self.node.connect('accessible-changed', self._onNodeUpdated)
+
+    self.main_xml.connect_signals(self)
     self.show_all()
 
   def _onStartStop(self):
@@ -89,6 +92,11 @@ class EventMonitor(ViewportPlugin):
   def _onClearlog(self):
     self.monitor_buffer.set_text('')
 
+  def _onNodeUpdated(self, node, acc):
+    if acc == node.desktop and \
+          self.source_filter in ('source_app', 'source_acc'):
+      self.monitor_toggle.set_active(False)
+
   def _popEventsModel(self):
     '''
     Populate the model for the event types tree view. Uses a constant
@@ -101,38 +109,11 @@ class EventMonitor(ViewportPlugin):
     events.sort()
     gobject.idle_add(self._appendChildren, None, '', 0, events)
 
-  def _initTreeView(self):
-    '''
-    Construct the data model and tree view columns for the event types tree.
-    '''
-    self.events_model = gtk.TreeStore(str,  # COL_NAME
-                                      str, # COL_FULL_NAME
-                                      bool, # COL_TOGGLE  
-                                      bool) # COL_INCONSISTENT
-    event_tree = self.main_xml.get_widget('treeview_events')
-    event_tree.set_model(self.events_model)
-    crt = gtk.CellRendererText()
-    crc = gtk.CellRendererToggle()
-    crc.connect('toggled', self._onToggled)
-    tvc = gtk.TreeViewColumn(_('Name'))
-    tvc.pack_start(crc, True)
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crc, 
-                       active=self.COL_TOGGLE,
-                       inconsistent=self.COL_INCONSISTENT)
-    tvc.set_attributes(crt, text=self.COL_NAME)
-    event_tree.append_column(tvc)
-    crt = gtk.CellRendererText()
-    tvc = gtk.TreeViewColumn(_('Full name'))
-    tvc.pack_start(crt, True)
-    tvc.set_attributes(crt, text=self.COL_FULL_NAME)
-    event_tree.append_column(tvc)
-
   def _initTextView(self):
     '''
     Initialize text view in monitor plugin.
     '''
-    self.textview_monitor = self.main_xml.get_widget('textview_monitor')
+    self.textview_monitor = self.main_xml.get_object('textview_monitor')
     
     self.monitor_buffer = self.textview_monitor.get_buffer()
     self.monitor_mark = \
@@ -356,8 +337,8 @@ class EventMonitor(ViewportPlugin):
 
   def _createHyperlink(self, acc):
     '''
-    Create a hyperlink tag for a given accessible. When the link is double 
-    clicked the accessible is selected in the main program.
+    Create a hyperlink tag for a given accessible. When the link is clicked
+    the accessible is selected in the main program.
     
     @param acc: The accessible to create the tag for.
     @type acc: Accessibility.Accessible
@@ -376,10 +357,9 @@ class EventMonitor(ViewportPlugin):
 
   def _onLinkClicked(self, tag, widget, event, iter):
     '''
-    Callback for doulbe clicked link. Select links accessible in main
-    application.
+    Callback for clicked link. Select links accessible in main application.
     
-    @param tag: Tag that was double clicked.
+    @param tag: Tag that was clicked.
     @type tag: gtk.TextTag
     @param widget: The widget that received event.
     @type widget: gtk.Widget
@@ -388,7 +368,8 @@ class EventMonitor(ViewportPlugin):
     @param iter: The text iter that was clicked.
     @type iter: gtk.TextIter
     '''
-    if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
+    if event.type == gtk.gdk.BUTTON_RELEASE and \
+           event.button == 1 and not self.monitor_buffer.get_has_selection():
       self.node.update(tag.get_data('acc'))
 
   def _onLinkKeyPress(self, textview, event):
@@ -517,10 +498,9 @@ class EventMonitor(ViewportPlugin):
     @rtype: boolean
     '''
     if self.source_filter == 'source_app':
-      if (hasattr(event.source, 'getApplication') and
-          hasattr(self.acc, 'getApplication')):
+      try:
         return event.source.getApplication() == self.acc.getApplication()
-      else:
+      except:
         return False
     elif self.source_filter == 'source_acc':
       return event.source == self.acc
